// Set up audio

sample_rate = 48000;
channels = 2;
bufsize = 512;

rtsetparams(sample_rate, channels, bufsize);

// Load all the instruments we need

load("CHAIN");
load("DMOVE");
load("MOOGVCF");
load("STRUM");
load("MPLACE");
load("WAVESHAPE");

// Set up the room for DMOVE and MPLACE

mikes(45.0, 0.5);
threshold(0.001);
randseed = 1;

srand(randseed);

dist_front=115.88
dist_right=127.2
dist_rear=-125
dist_left=-126.8
height=105.3
rvbtime=1.5
abs_fac=5

space(dist_front,dist_right,dist_rear,dist_left,height/4,abs_fac,rvbtime)
dist_mikes = 5;

// Route instruments:  CHAIN(START -> MOOGVCF -> MPLACE) -> RVB

bus_config("START","chain 0 out");
bus_config("MOOGVCF", "chain 0 in", "chain 1 out");
bus_config("MPLACE", "chain 1 in", "chain 2-5 out");
bus_config("CHAIN", "aux 0-3 out");
bus_config("RVB","aux 0-3 in","out 0-1");

// Utility for picking values out of an array in such a way that we end up
// using all of the values.

float chooseAndClear(list inputArray, list slots)
{
	length = len(inputArray);
	index = trand(0, length);
	count = 0;
	while (slots[index] == 1) { 
		index = trand(0, length);
		if (count > 2 * length) {
			for (i = 0; i < length; i += 1) slots[i] = 0;	// reset all slots
		}
		count += 1;
	}
	slots[index] = 1;
	value = inputArray[index];
	return value;
}

global_cutoff_factor = 1;

// Function which acts as compound instrument, chaining START, MOOGVCF, and MPLACE
// 'rho' and 'theta' are the distance and angle to this note

float fstrum(float outskip, float dur, float pitch, float intensity, float rho, float theta)
{
	// randomly vary the character of the strum sound
	fund_decay = irand(dur * 0.4, dur * 0.6);
	nf_decay = irand(0.01, 0.08);
	// filter cutoff depends on "intensity"
	cf = 60 + (5000 * intensity / 10);
	cf *= global_cutoff_factor;
	res = 0.05;
	amp = 10000 + (20000 * intensity / 10);
	handle start;	// this is so we don't accidentally override 'start' in the main score!
	start = makeinstrument("START", 0, dur, pitch, fund_decay, nf_decay, amp, 1, 0);
	moog = makeinstrument("MOOGVCF", 0, 0, dur, 20, inchan=0, pan=0, bypass=0, cf, res)
	place = makeinstrument("MPLACE", 0,0,dur,12,rho,theta,dist_mikes);
	return CHAIN(outskip, dur, 3, start, moog, place);
}

float sum(list array)
{
	total = 0;
	for (i = 0; i < len(array); i += 1) {
		total += array[i];
	}
	return total;
}

// Which hexachord do we start on.  It switches every subsection.
c_hexachord = 1;
// How many measures in each sub-section of the piece.  The pairs each sum to 12.
measure_counts = { 8,4, 7,5, 6,6, 5,7, 4,8, 3,9 };
count_index = 0;

// Notes for the C hexachord
c_array = { 0.00, 0.02, 0.04, 0.05, 0.07, 0.09 };
// Slots used with chooseAndClear(), above
c_slots = { 0, 0, 0, 0, 0, 0 };
// Notes for the F-sharp hexachord
fs_array = { 0.06, 0.08, 0.10, 0.11, 1.01, 1.03 };
fs_slots = { 0, 0, 0, 0, 0, 0 };

// Articulations of beats: 0 = rest, 2 = accented, 1 = non-accented.
// Grouped in beat pattern of 3, 4, 5 with rest as first for each.
beats = { 0,2,1, 0,2,1,1, 0,2,1,1,1 };
numbeats = len(beats);
beat = 0;
soundingbeat = 0;
measure = 0;

start = 1;

// Distance between beats in seconds
incr = 0.15;

num_measures = sum(measure_counts);

// accompaniment

for (notes = 0; notes < num_measures * numbeats; ++notes) {
	if (beats[beat]) {
		// Set up "accents" in accompaniment using 'strength' as a gain adjustment
		float strength;
		if (beats[beat] == 2) { strength = 9; }
		else { strength = 4 }
		dur = 1;
		// C hexachord plays two notes at a time from the hexachord pitches.
		// The 'c_array' and 'fs_array' lists just have the fractional (half-step) part, so we add the octave (8).
		// The angles to the sounds progress each beat, 180 degrees opposite each other
		if (c_hexachord == 1) {
			distance = 20;
			fstrum(start, dur, 8 + chooseAndClear(c_array, c_slots), strength, distance, 360 * soundingbeat / numbeats);
			fstrum(start, dur, 8 + chooseAndClear(c_array, c_slots), strength, distance, 180 + (360 * soundingbeat / numbeats));
		}
		// F-sharp hexachord plays three notes at a time from the hexachord pitches.  Start times are slightly randomized.
		// The angles to the sounds progress each beat, each 120 degrees ahead of the other.
		else {
			distance = 15;
			// Make these a bit softer since there are 3 at a time.
			strength *= 0.6;	
			fstrum(start+irand(-0.005, 0.005), dur, 8 + chooseAndClear(fs_array, fs_slots), strength, distance, 120 * soundingbeat / numbeats);
			fstrum(start+irand(-0.005, 0.005), dur, 8 + chooseAndClear(fs_array, fs_slots), strength, distance, 240 + (360 * soundingbeat / numbeats));
			fstrum(start+irand(-0.005, 0.005), dur, 8 + chooseAndClear(fs_array, fs_slots), strength, distance, 0 + (360 * soundingbeat / numbeats));
		}
		soundingbeat += 1;
	}
	// When we hit the number of beats in this measure, advance measures
	beat = (beat + 1) % numbeats;
	if (beat == 0) {
		measure += 1;
	}
	// When we hit the number of measures in this section, reset and switch hexachords
	if (measure == measure_counts[count_index]) {
		count_index = (count_index + 1) % len(measure_counts);
		measure = 0;
		c_hexachord = (c_hexachord == 0);
	}
	start += incr;
}

// Store the offset to the end of the above
acomp_end = start;

// Set up the bass part - it starts at an offset into the accompaniment
bstart = incr * 2 * numbeats;
bindex = 0;

// Bass part plays a regular pattern of 5, 4, and 3 beats throughout (compare to 3,4,5 pattern of accompaniment)
basslength = { 5, 4, 3 };
numbasslength = len(basslength);
measure = 2;
count_index = 0;

global_cutoff_factor = 0.46;		// dull down the bass

// bass
while (bstart < acomp_end-1) {
	dur = 2;
	if (c_hexachord) {
		fstrum(bstart, dur, 6 + chooseAndClear(c_array, c_slots), 9, 20, 360 * bindex / numbeats);
	}
	else {
		fstrum(bstart, dur, 5 + chooseAndClear(fs_array, c_slots), 5, 15, 180 * bindex / numbeats);
	}
	bstart += basslength[bindex] * incr;
	// When we hit the number of beats in this measure, advance measures
	bindex = (bindex + 1) % numbasslength;
	if (bindex == 0) {
		measure += 1;
	}
	// When we hit the number of measures in this section, reset and switch hexachords
	if (measure == measure_counts[count_index]) {
		count_index = (count_index + 1) % len(measure_counts);
		measure = 0;
		c_hexachord = (c_hexachord == 0);
	}
}

// Set up the melody

// New routing for WAVESHAPE instrument

bus_config("WAVESHAPE", "chain 0 out");
bus_config("MPLACE", "chain 0 in", "chain 2-5 out");

// More setup for WAVESHAPE
wavetable = maketable("wave", 10000, "sine")
transferfunc = maketable("cheby", 10000, 0.9, 0.3, -0.2, 0.6, -0.7)
indexguide = maketable("line", 10000, 0,0, 2.5,1, 7,0)

viberamp = maketable("line", 10000, 0, 0, 1, 0.2, 3, 1, 5, 0);
vibe = makeLFO("tri", 3.8, 0.007)

// Melody comes from fixed distance and angle
rho = 30;
theta = 24;
amp = 30000;
env = maketable("line", 10000, 0,0, 1,1, 3, 1, 7,0);

wstart = numbeats * incr * 4;
oldpitch = 0;

while (wstart < acomp_end - 3) {
	float pitch;
	float in_c, total;
	in_c = 0;
	for (m = 0; m < len(measure_counts) && total < wstart; ++m) {
		total += measure_counts[m] * numbeats * incr;
		in_c = (in_c == 0);
	}
	// Pick melodic pitches based on which hexachord we are in.
	if (in_c) {
		pitch = pickrand(9.11, 9.04, 9.05, 9.02, 9.07);
		// Don't allow repetition of previous pitch if possible
		if (pitch == oldpitch) {
			pitch = pickrand(9.11, 9.04, 9.05, 9.02, 9.07);
		}
		oldpitch = pitch;
	}
	else {
		pitch = pickrand(10.01, 9.05, 9.10, 9.08, 9.03);		// 6, 8, 10, 11, 13
		if (pitch == oldpitch) {
			pitch = pickrand(10.01, 9.05, 9.10, 9.08, 9.03);
		}
		oldpitch = pitch;
	}
	// Melody notes are either 3 or 4 beats long, or 12 beats long
	rbeats = trunc(pickrand(pickrand(3, 3, 3, 3, 4, 4, 4), 12));
	skip = rbeats * incr;
	dur = skip + 1;
	// chance() returns true 2 out of 3 times here - else we get a rest in the melody.
	if (chance(2, 3)) {
		shape = makeinstrument("WAVESHAPE", 0, dur, pitch + (vibe * viberamp), 0, 1, amp * env, pan=0, wavetable, transferfunc, indexguide)
		place = makeinstrument("MPLACE", 0,0,dur,100,rho,theta,dist_mikes);
		CHAIN(wstart, dur, 2, shape, place);
	}
	// Location of melody randomly flips between 24 degrees and -24 degrees.
	theta *= pickrand(-1, 1);
	wstart += skip;
}


end_time = start+2;
RVB(0, 0, end_time+4, 0.2);
